import requests
import urllib.request
from bs4 import BeautifulSoup
import numpy as np
import matplotlib.pyplot as plt

import os
import re
import time
from datetime import date, timedelta


PTT_URL = 'https://www.ptt.cc'


def get_web_page(url):
    time.sleep(0.5)  # 每次爬取前暫停 0.5 秒以免被 PTT 網站判定為大量惡意爬取
    resp = requests.get( url=url, cookies={'over18': '1'})
    if resp.status_code != 200:
        print('Invalid url:', resp.url)
        return None
    else:
        return resp.text


def get_articles(dom, target_date):
    soup = BeautifulSoup(dom, 'html.parser')

    # 取得上一頁的連結
    paging_div = soup.find('div', 'btn-group btn-group-paging')
    prev_url = paging_div.find_all('a')[1]['href']

    articles = []  # 儲存取得的文章資料
    divs = soup.find_all('div', 'r-ent')
    for d in divs:
        if d.find('div', 'date').string.strip() == target_date:  # 發文日期正確
            # 取得推文數
            push_count = 0
            if d.find('div', 'nrec').text:
                try:
                    push_count = int(d.find('div', 'nrec').text)  # 轉換字串為數字
                except ValueError:  # 若轉換失敗，不做任何事，push_count 保持為 0
                    pass
            # 取得文章連結及標題
            if d.find('a'):  # 有超連結，表示文章存在，未被刪除
                href = d.find('a')['href']
                title = d.find('a').text
                articles.append({'title': title, 'href': href, 'push_count': push_count})
    return articles, prev_url


def parse(dom):
    soup = BeautifulSoup(dom, 'html.parser')
    links = soup.find(id='main-content').find_all('a')
    img_urls = []
    for link in links:
        if re.match(r'^https?://(i.)?(m.)?imgur.com', link['href']):
            img_urls.append(link['href'])
    return img_urls


def save(img_urls, title):
    if img_urls:
        try:
            dname = title.strip()  # 用 strip() 去除字串前後的空白
            os.makedirs(dname)
            for img_url in img_urls:
                fname = img_url.split('/')[-1]
                urllib.request.urlretrieve(img_url, os.path.join(dname, fname))
        except Exception as e:
            print(e)


# -------------------------------爬圖片
current_page = get_web_page(PTT_URL + '/bbs/Beauty/index.html')
if current_page:
	articles = []  # 全部的今日文章
	
	for i in range(2):
		target_date = (date.today() - timedelta(i)).strftime("%m/%d").lstrip('0')  		# 指定日期, 去掉開頭的 '0' 以符合 PTT 網站格式
		current_articles, prev_url = get_articles(current_page, target_date)  			# 目前頁面的指定日期文章
		while current_articles:  														# 若目前頁面有指定日期文章則加入 articles，並回到上一頁繼續尋找是否有其他指定日期文章
			articles += current_articles
			current_page = get_web_page(PTT_URL + prev_url)
			current_articles, prev_url = get_articles(current_page, target_date)

	# 已取得文章列表，開始進入各文章讀圖
	for article in articles:
		print('讀取中...', article)
		page = get_web_page(PTT_URL + article['href'])
		if page:
			img_urls = parse(page)
			save(img_urls, article['title'])
			article['num_image'] = len(img_urls)
				
# -------------------------------資料分析
images = [] # 圖片數
pushes = [] # 推文數
for article in articles:
	images.append(article['num_image'])
	pushes.append(article['push_count'])
print('圖片數:', images)
print('Max:', max(images), 'Min:', min(images), '平均:', sum(images)/len(images))
print('推文數:', pushes)
print('Max:', max(pushes), 'Min:', min(pushes), '平均:', sum(pushes)/len(pushes))
print('相關係數:', np.corrcoef(images, pushes)[1,0])

# -------------------------------資料視覺化
# 將推文數和圖片數的關聯視覺化
plt.scatter(pushes, images)
plt.title('pushes vs. images')
plt.xlabel('# of pushes')
plt.ylabel('# of images')
plt.show()

# 將推文數(以5做為區間)視覺化
n = int(max(pushes)/5)+1

x = []
for i in range(n):
	x.append(5*i-4)
y = []
for i in range(n):
	y.append(0)
for push in pushes:			
	y[int(push/5)] += 1	# 將推文數五分位化

plt.bar(x, y, 8, facecolor='green')
plt.title('pushes vs. posts')
plt.xlabel('# of pushes')
plt.ylabel('# of posts')
plt.axis([-5, max(pushes)+5, 0, max(y)])
plt.xticks([5 * i for i in range(n)])
plt.show()